Package: STATegRa
Type: Package
Title: Classes and methods for multi-omics data integration
Version: 1.48.0
Date: 2018-05-02
Author: STATegra Consortia
Maintainer: David Gomez-Cabrero <david.gomezcabrero@ki.se>, Núria Planell <nuria.planell.picola@navarra.es>
Depends: R (>= 2.10)
Imports: Biobase, gridExtra, ggplot2, methods, stats, grid, MASS,
        calibrate, gplots, edgeR, limma, foreach, affy
Suggests: RUnit, BiocGenerics, knitr (>= 1.6), rmarkdown, BiocStyle (>=
        1.3), roxygen2, doSNOW
VignetteBuilder: knitr
Encoding: UTF-8
biocViews: Software, StatisticalMethod, Clustering, DimensionReduction,
        PrincipalComponent
Description: Classes and tools for multi-omics data integration.
License: GPL-2
LazyLoad: yes
Collate: 'STATegRa_combiningMappings.R' 'STATegRa_fused.R'
        'STATegRa_holistOmics.R' 'STATegRa_omicsCLUST_bioMap.R'
        'STATegRa_omicsCLUST_bioDist.R'
        'STATegRa_omicsCLUST_bioDistW.R' 'STATegRa_omicsNPC.R'
        'STATegRa_omicsNPC_internal.R' 'STATegRa_omicsPCA_caClass.R'
        'STATegRa_omicsPCA_methods.R' 'STATegRa_omicsPCA_plotting.R'
        'STATegRa_package.R'
RoxygenNote: 5.0.1
git_url: https://git.bioconductor.org/packages/STATegRa
git_branch: RELEASE_3_23
git_last_commit: e70783e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:02:22 UTC; biocbuild
